/***************************************************************************
 *   Copyright (C) 2010 Thomas Keller                                      *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#ifndef ANNOTATE_H
#define ANNOTATE_H

#include <QAbstractItemModel>
#include <QDate>
#include <QFont>
#include <QBrush>
#include <QStack>
#include "MonotoneProcess.h"
#include "Certs.h"
#include "vocab.h"

class Annotate;

class Annotation
{
public:
    Annotation(Annotate *, QString, QString, bool);
    QVariant data(int, int) const;

private:
    Annotate * model;
    QString revision;
    QString code;
    bool show_info;

    QFont annotationFont;
    QFont codeFont;

    friend class Annotate;
};

class Annotate : public QAbstractItemModel
{
    Q_OBJECT
public:
    Annotate(QObject *);
    virtual ~Annotate();
    CertList getCachedCertList(const QString &) const;

    // needed Qt Model methods
    QVariant data(const QModelIndex &, int) const;
    Qt::ItemFlags flags(const QModelIndex &) const;
    QVariant headerData(int, Qt::Orientation, int) const;
    QModelIndex index(int, int, const QModelIndex &) const;
    QModelIndex parent(const QModelIndex &) const;
    int rowCount(const QModelIndex &) const;
    int columnCount(const QModelIndex &) const;

public slots:
    void readAnnotation(const MonotoneHandlePtr &, const QString &, const QString &);
    void setSelectedRevision(const QString &);
    QString getSelectedRevision() const;
    QString getBaseRevision() const;

signals:
    void annotationRead();
    void currentCertList(const CertList &);

private:
    MonotoneHandlePtr monotoneHandle;

    MonotoneProcess * proc;
    Certs * certsModel;

    QList<Annotation> annotationLines;
    QStack<QString> certsToRead;
    QMap<QString, CertList> certMap;
    QMap<QString, QBrush> brushMap, inverseBrushMap;

    QBrush highlightBrush;

    QString selectedRevision;
    QString baseRevision;

    void readNextCerts();

private slots:
    void processFinished(int exitCode, QProcess::ExitStatus exitStatus);
    void certsRead(const CertList &);

    friend class Annotation;
};

#endif
