/***************************************************************************
 *   Copyright (C) 2007 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#ifndef AUTOMATE_COMMAND_H
#define AUTOMATE_COMMAND_H

#include "MonotoneThread.h"

#include <QString>
#include <QList>
#include <QMutex>

class AutomateCommandHelper;

class AutomateCommand
{
public:
    AutomateCommand(AutomateCommandHelper * helper = 0);
    virtual ~AutomateCommand();
    friend class AutomateCommandHelper;

protected:
    virtual void processTaskResult(const MonotoneTaskPtr &) = 0;
    virtual void processTaskAbortion(const MonotoneTaskPtr &);
    virtual void tickerUpdate(const TickerMap &);
    void enqueueTask(const MonotoneHandlePtr &, const MonotoneTaskPtr &);
    bool hasQueuedCommands() const;
    void abortThreads();

private:
    QMutex lock;
    QList<int> connectedThreads;
    QMap<int, QList<int> > queuedCommands;
    AutomateCommandHelper * cmdHelper;
    bool cmdHelperCreated;
};

// Qt's class hierarchy doesn't support multiple inheritance, so we need
// a helper object which manages our signal/slot connections
class AutomateCommandHelper : public QObject
{
    Q_OBJECT
public:
    AutomateCommandHelper(AutomateCommand *);

protected slots:
    virtual void taskAborted(const MonotoneTaskPtr &);
    void taskFinished(const MonotoneTaskPtr &);
    void tickersChanged(const TickerMap &);

private:
    AutomateCommand * command;
};

#endif
