/***************************************************************************
 *   Copyright (C) 2010 Thomas Keller                                      *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#include "BranchesProxyModel.h"
#include "vocab.h"

BranchesProxyModel::BranchesProxyModel(QObject * parent)
  : QSortFilterProxyModel(parent), showSuspended(false)
{
    // FIXME: maybe this should be configurable?
    filter.setCaseSensitivity(Qt::CaseSensitive);
    filter.setPatternSyntax(QRegExp::Wildcard);
}

BranchesProxyModel::~BranchesProxyModel() {}

bool BranchesProxyModel::filterAcceptsRow(int sourceRow, const QModelIndex & sourceParent) const
{
    QModelIndex sourceIndex = sourceModel()->index(sourceRow, 0, sourceParent);
    bool acceptRow = true;
    acceptRow &= showSuspended || !sourceIndex.data(Qt::UserRole).toBool();
    acceptRow &= filter.isEmpty() || indexMatchesFilter(sourceIndex);

    return acceptRow;
}

void BranchesProxyModel::showSuspendedBranches(bool show)
{
    if (show == showSuspended)
        return;

    showSuspended = show;
    invalidate();
}

void BranchesProxyModel::filterByName(const QString & pattern)
{
    filter.setPattern(pattern);
    invalidate();
}

// FIXME: we're basically doing this twice for each proxy model, while
// a single proxy model which does the filtering for both (sub) proxy
// models should be enough. Tweak that some time.
bool BranchesProxyModel::indexMatchesFilter(const QModelIndex & index) const
{
    QString branch = index.data(Qt::ToolTipRole).toString();
    if (filter.indexIn(branch) >= 0)
    {
        return true;
    }

    // we always have to filter _all_ items of the view even if we only
    // display a subset of them in a list-alike way, since the parents
    // of a specific matched node must match as well in order to keep
    // the node in
    int childCount = -1;
    while (true)
    {
        QModelIndex child = index.child(++childCount, 0);
        if (!child.isValid())
            break;

        if (indexMatchesFilter(child))
            return true;
    }

    return false;
}

