/***************************************************************************
 *   Copyright (C) 2010 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#ifndef CERT_LIST_H
#define CERT_LIST_H

#include "vocab.h"
#include <QDateTime>

struct Cert
{
    enum Trust { Trusted, Untrusted } trust;
    enum Signature { Ok, Bad, Unknown } signature;
    QString key;
    QString name;
    QString value;

    QDateTime dateTimeValue() const;
    QString plainValue() const;
    QString htmlValue() const;
};

struct CertList
{
    QString revision;
    QList<Cert> certs;

    void add(const Cert & c);
    int size() const;
    Cert at(int pos) const;
    void clear();

    QStringList signers() const;
    QList<Cert> findBySigner(const QString & key = QString()) const;
    QList<Cert> findByName(const QString & name,
                           const QString & key = QString()) const;
    QStringList findByNameValues(const QString & name,
                                 const QString & key = QString()) const;
    void fill(const StanzaList &);
};

#endif
