/***************************************************************************
 *   Copyright (C) 2007 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#ifndef GETCONTENTCHANGED_H
#define GETCONTENTCHANGED_H

#include "AutomateCommand.h"

#include <QAbstractItemModel>
#include <QQueue>
#include <QMap>

class GetContentChanged : public QAbstractItemModel, public AutomateCommand
{
    Q_OBJECT
public:
    GetContentChanged(QObject *, const MonotoneHandlePtr &);
    virtual ~GetContentChanged();

    // needed Qt Model methods
    QVariant data(const QModelIndex &, int) const;
    Qt::ItemFlags flags(const QModelIndex &) const;
    QVariant headerData(int, Qt::Orientation, int) const;
    QModelIndex index(int, int, const QModelIndex &) const;
    QModelIndex parent(const QModelIndex &) const;
    int rowCount(const QModelIndex &) const;
    int columnCount(const QModelIndex &) const;

public slots:
    void readChanges(const QString &, const QString &);
    void stopReading();

signals:
    // is emitted each time the algorithm reaches the end of a
    // particular development line for a specific file
    void endOfLineReached();
    // is emitted if the root revision has been reached,
    // which has per se no parent revisions
    void rootReached();

private:
    void queryContentChanged(const QString &, const QString &);
    void queryParents(const QString &);
    void queryCorrespondingPath(const QString &, const QString &, const QString &);

    void processTaskResult(const MonotoneTaskPtr &);

    QStringList revisions;

    QQueue<QString> revsForPathStack;

    QMap<QString, QString> pathInRevision;

    QString startRev;
    QString startPath;

    MonotoneHandlePtr monotoneHandle;
    bool stopReadingChanges;
};

#endif

