/***************************************************************************
 *   Copyright (C) 2008 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#ifndef INVENTORY_H
#define INVENTORY_H

#include "AutomateCommand.h"
#include "InventoryItem.h"

class InventoryModel;

class Inventory : public QObject, public AutomateCommand
{
    Q_OBJECT

    friend class InventoryModel;

public:
    Inventory(QObject *);
    ~Inventory();

    QMap<FileEntry, FileEntryList> findUnaccountedRenames() const;
    static const int QueryLevel;

public slots:
    void setMonotoneHandle(const MonotoneHandlePtr &);
    void read(const QString & path = QString(), bool fullQuery = false);

private:
    void processTaskResult(const MonotoneTaskPtr &);
    void insertRowsRecursive(ModelItem *, const QMap<ModelItem *, QList<ModelItem *> > &);
    void removeRowsRecursive(ModelItem *, int, int);

    ModelItem * rootItem;
    QMap<QString, InventoryItem *>  itemMap;
    MonotoneHandlePtr monotoneHandle;

signals:
    void beginInsertRows(ModelItem *, int, int);
    void endInsertRows();

    void beginRemoveRows(ModelItem *, int, int);
    void endRemoveRows();

    void dataChanged(ModelItem *);

    void inventoryRead(const QString &);
    void invalidWorkspaceFormat();
};

#endif

