/***************************************************************************
 *   Copyright (C) 2006 by Ingo Maindorfer								   *
 *   ingo@liquidcooling.de												   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#ifndef INVENTORYPROXYMODEL_H
#define INVENTORYPROXYMODEL_H

#include "InventoryItem.h"
#include <QSortFilterProxyModel>

class InventoryProxyModel : public QSortFilterProxyModel
{
    Q_OBJECT
public:
    enum ViewOption { All = 0, Changed, ContentsChanged, AttributesChanged,
                      Added, Dropped, Renamed, Missing, Unknown, Ignored,
                      Invalid };

    InventoryProxyModel(QObject *, bool);
    ~InventoryProxyModel();
    bool filterAcceptsRow(int, const QModelIndex &) const;
    bool lessThan(const QModelIndex &, const QModelIndex &) const;
    void sort(int, Qt::SortOrder order = Qt::AscendingOrder);

    static QModelIndex inventoryIndex(const QModelIndex &);

public slots:
    void setHideIgnoredFiles(bool);
    void setViewOption(int);
    void filterItemsByName(const QString &);

private:
    bool itemMatchesFilter(const InventoryItem *) const;

    bool folderTree;
    int sortColumn;
    bool hideIgnored;
    ViewOption viewOption;
    Qt::SortOrder sortOrder;
    QRegExp filter;
};

#endif
