/***************************************************************************
 *   Copyright (C) 2008 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#ifndef INVENTORY_WATCHER_H
#define INVENTORY_WATCHER_H

#include "vocab.h"
#include "LRUCache.h"

#include <QFileSystemWatcher>
#include <QModelIndex>
#include <QTimer>

class InventoryWatcher : public QFileSystemWatcher
{
    Q_OBJECT
public:
    InventoryWatcher(QObject *);
    ~InventoryWatcher();

signals:
    void changedPath(const QString &);
    void changedBaseRevision(const QString &);

public slots:
    void setMonotoneHandle(const MonotoneHandlePtr &);
    void watchPaths(const QStringList &);
    void unwatchPaths(const QStringList &);
    void clearAllWatches();

private:
    void markPathForNotification(const QString &);
    void checkForBookkeepChanges();

    MonotoneHandlePtr monotoneHandle;
    StanzaList oldRevisionEntries;
    QStringList changedPaths;
    QTimer * notifyTimer;
    LRUCache<QString> lruCache;

private slots:
    void pathChanged(const QString &);
    void notifyChangedPaths();
};

#endif

