/***************************************************************************
 *   Copyright (C) 2006 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#ifndef KEYS_H
#define KEYS_H

#include "AutomateCommand.h"

#include <QAbstractItemModel>

struct Key {
    static const int Database;
    static const int Keystore;
    QString given_name;
    QString local_name;
    QString hash;
    int public_locations;
    int private_locations;
    Key() : public_locations(0), private_locations(0) {};
    bool isPrivate() { return (private_locations & Keystore) == Keystore; }
};

class Keys : public QAbstractItemModel, public AutomateCommand
{
    Q_OBJECT
public:
    Keys(QObject *, const MonotoneHandlePtr &);
    virtual ~Keys();

    // needed Qt Model methods
    QVariant data(const QModelIndex &, int) const;
    Qt::ItemFlags flags(const QModelIndex &) const;
    QVariant headerData(int, Qt::Orientation, int) const;
    QModelIndex index(int, int, const QModelIndex &) const;
    QModelIndex parent(const QModelIndex &) const;
    int rowCount(const QModelIndex &) const;
    int columnCount(const QModelIndex &) const;

public slots:
    void readKeys();

signals:
    void keysRead();

private:
    void processTaskResult(const MonotoneTaskPtr &);
    QString getLocationString(int) const;
    QString getPasswordSaveStatus(Key *) const;
    QList<Key *> keys;
    MonotoneHandlePtr monotoneHandle;
};

#endif

