/***************************************************************************
 *   Copyright (C) 2007 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#ifndef MANIFEST_H
#define MANIFEST_H

#include "AutomateCommand.h"

#include <QAbstractItemModel>
#include <QList>
#include <QMap>

struct ManifestEntry
{
    ManifestEntry * parent;
    QList<ManifestEntry*> children;
    QString path;
    QString hash;
    bool is_directory;
    QMap<QString, QString> attributes;

    inline ManifestEntry() : parent(NULL), path(""), hash(""), is_directory(true) {}
    inline ~ManifestEntry()
    {
        foreach(ManifestEntry *it, children)
            delete it;
    }

    inline QString filename()
    {
        int last = path.lastIndexOf("/");
        if (last == -1) return path;
        return path.right(path.size() - last - 1);
    }
    inline int row()
    {
        if (parent)
            return parent->children.indexOf(const_cast<ManifestEntry *>(this));
        return 0;
    }
    inline ManifestEntry * child(int row)
    {
        return children.value(row);
    }
    inline int childCount()
    {
        return children.size();
    }
};

class Manifest : public QAbstractItemModel, public AutomateCommand
{
    Q_OBJECT
public:
    Manifest(QObject *, const MonotoneHandlePtr &);
    virtual ~Manifest();

    // needed Qt Model methods
    QVariant data(const QModelIndex &, int) const;
    Qt::ItemFlags flags(const QModelIndex &) const;
    QVariant headerData(int, Qt::Orientation, int) const;
    QModelIndex index(int, int, const QModelIndex &) const;
    QModelIndex parent(const QModelIndex &) const;
    int rowCount(const QModelIndex &) const;
    int columnCount(const QModelIndex &) const;

public slots:
    void readManifest(const QString &);

signals:
    void manifestRead();

private:
    void processTaskResult(const MonotoneTaskPtr &);
    MonotoneHandlePtr monotoneHandle;
    ManifestEntry * root;
};

#endif

