/***************************************************************************
 *   Copyright (C) 2007 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#include "FileExporter.h"
#include "MonotoneUtil.h"

//
// FIXME: remove these two dependencies:
// a) refactor this class into a QThread
// b) use signals to let the GUI thread now the state of the action
//
#include "GuitoneCore.h"
#include <QProgressDialog>

FileExporter::FileExporter(const MonotoneHandlePtr & database,
                           const QString & rev, const QDir & dir)
  : QObject(0), monotoneHandle(database), rootDir(dir), revision(rev)
{
    I(rootDir.exists());
}

FileExporter::~FileExporter() {}

bool FileExporter::run(const FileEntryList & entries)
{
    QProgressDialog progress(
        tr("Processing files..."), tr("Abort"), 0, entries.size()
    );
    progress.setWindowModality(Qt::WindowModal);

    bool success = true;
    bool cleanup = false;
    for (int i=0, j=entries.size(); i < j; i++)
    {
        if (!exportFile(entries.at(i)))
        {
            success = false;
            cleanup = true;
            break;
        }

        APP->processEvents();
        if (progress.wasCanceled())
        {
            cleanup = true;
            break;
        }
        progress.setValue(i + 1);
    }

    if (cleanup)
    {
        D("TODO: clean up written files");
    }
    return success;
}

bool FileExporter::exportFile(const FileEntry & entry)
{
    if (entry.is_dir)
    {
        if (!entry.path.isEmpty() && !rootDir.mkpath(entry.path))
        {
            C(QString("Cannot create directory '%1'").arg(entry.path));
            return false;
        }
        return true;
    }

    MonotoneTaskPtr task(new MonotoneTask(
        QStringList() << "get_file_of" << entry.path,
        QStringList() << "r" << revision
    ));

    MonotoneUtil::runSynchronousTask(monotoneHandle, task);

    if (!task->isFinished()) F("task aborted");

    if (task->getReturnCode() > 0)
    {
        C(QString("get_file_of '%1' (rev: %2) failed: %3")
            .arg(entry.path).arg(revision).arg(task->getDecodedOutput()));
        return false;
    }

    QFile file(rootDir.filePath(entry.path));
    if (file.exists()) file.remove();

    if (!file.open(QIODevice::WriteOnly))
    {
        C(QString("Can't open '%1' for writing").arg(entry.path));
        return false;
    }

    I(file.write(task->getOutput()));

    if (entry.attrs.contains("mtn:execute") &&
        entry.attrs.value("mtn:execute") == "true")
    {
        file.setPermissions(file.permissions() | QFile::ExeOwner | QFile::ExeUser);
    }

    file.close();

    return true;
}

