/***************************************************************************
 *   Copyright (C) 2010 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#ifndef MONOTONE_HANDLE_H
#define MONOTONE_HANDLE_H

#include "vocab.h"

class MonotoneHandle
{
public:
    typedef enum { empty_handle = 1,
                   workspace_handle,
                   database_handle,
                   server_handle } Type;

    Type getType() const { return type; }

    QString getData() const { return data; }

    QString identify() const;

    static MonotoneHandlePtr create(const QString & handle = QString());

private:
    Type type;
    QString data;

    MonotoneHandle(Type, const QString &);

    static void validateWorkspacePath(QString &, QString &);
    static void validateDatabaseFile(const QString &);
    static void validatePrivateKey(const QString &, QString &);
    static void validateServerConnection(const QString &);
    static void resolveDatabaseAlias(QString &);

    static QStringList defaultDatabaseLocations;
    static QMap<QString, QString> privateKeys;
};

#endif
