/***************************************************************************
 *   Copyright (C) 2007 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#ifndef MONOTONE_MANAGER_H
#define MONOTONE_MANAGER_H

#include "MonotoneThread.h"

#include <QMultiMap>
#include <QSet>

class MonotoneManager : public QObject
{
    Q_OBJECT
public:
    MonotoneManager();
    ~MonotoneManager();

    //! set the path to the monotone binary to use for all new threads
    bool setMonotoneBinaryPath(const QString &, bool, QString &);

    //! returns if the path to the monotone binary was set successful before
    inline bool monotoneBinaryPathSet() const { return !mtnPath.isEmpty(); }

    //! returns an appropriate MonotoneThread for the given handle
    MonotoneThreadPtr getThread(const MonotoneHandlePtr &);

    //! stops the thread with the given thread number
    void stopThread(int);

    //! stops all threads for the given handle
    void stopThreads(const MonotoneHandlePtr &);

    //! returns the interface version of a given monotone binary
    static QString getInterfaceVersion(const QString &);

    //! compares two version strings
    static int versionCompare(const QString &, const QString &);

    //! the minimum required interface version needed
    static const QString MinInterfaceVersion;
    //! the maximum allowed interface version (excluding this particular version)
    static const QString MaxInterfaceVersion;

signals:
    //! is emitted if a thread is aborted and contains some diagnostic message
    void threadAborted(const QString &);

private:
    void removeThread(int);

    QMap<int, MonotoneThreadPtr> threadMap;
    QMultiMap<int, QString> identMap;
    QSet<int> stoppingThreads;

    QString mtnPath;
    int threadNumber;
    QMutex lock;

private slots:
    void stopped(int);
    void aborted(int, QProcess::ProcessError, const QString &);
};

#endif

