/***************************************************************************
 *   Copyright (C) 2010 by Thomas Keler                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#include "MonotoneTask.h"
#include <QTextCodec>

MonotoneTask::MonotoneTask()
{
    init(ByteArrayList(), ByteArrayList());
}

MonotoneTask::MonotoneTask(const MonotoneTask & other)
{
    returnCode = other.returnCode;
    commandNumber = other.commandNumber;
    threadNumber = other.threadNumber;
    finished = other.finished;
    arguments = other.arguments;
    options = other.options;
    output = other.output;
    abortTask = other.abortTask;
    outputEncoding = other.outputEncoding;
}

MonotoneTask::MonotoneTask(const QStringList & args)
{
    init(stringToByteArrayList(args), ByteArrayList());
}

MonotoneTask::MonotoneTask(const QStringList & args, const QStringList & opts)
{
    init(stringToByteArrayList(args), stringToByteArrayList(opts));
}

MonotoneTask::MonotoneTask(const ByteArrayList & args)
{
    init(args, ByteArrayList());
}

MonotoneTask::MonotoneTask(const ByteArrayList & args, const ByteArrayList & opts)
{
    init(args, opts);
}

void MonotoneTask::init(const ByteArrayList & args, const ByteArrayList & opts)
{
    arguments = args;
    options = opts;
    returnCode = -1;
    finished = false;
    abortTask = false;
    outputEncoding = "UTF-8";
}

ByteArrayList MonotoneTask::stringToByteArrayList(const QStringList & list)
{
    ByteArrayList byteArrayList;
    foreach (QString entry, list)
    {
        byteArrayList.append(entry.toUtf8());
    }
    return byteArrayList;
}

QByteArray MonotoneTask::getEncodedInput() const
{
    QByteArray commandLine;

    if (options.size() > 0)
    {
        // currently mtn can only understand key => value option pairs
        I(options.size() % 2 == 0);

        commandLine.append('o');
        for (int i=0, c=options.size(); i<c; i++)
        {
            QByteArray fragment = options.at(i);
            commandLine.append(QString::number(fragment.size()).toAscii());
            commandLine.append(':');
            commandLine.append(fragment);
        }
        // separate options from the command by a single whitespace
        commandLine.append("e ");
    }

    I(arguments.size() > 0);

    commandLine.append('l');
    for (int i=0, c=arguments.size(); i<c; i++)
    {
        QByteArray fragment = arguments.at(i);
        commandLine.append(QString::number(fragment.size()).toAscii());
        commandLine.append(':');
        commandLine.append(fragment);
    }
    commandLine.append('e');

    return commandLine;
}

QString MonotoneTask::getDecodedOutput(const QString & enc) const
{
    QString encoding = enc;
    if (encoding.isEmpty())
    {
        encoding = outputEncoding;
    }
    I(!encoding.isEmpty());

    // QTextCodec implements codecForName as factory and caches multiple
    // requests, so we don't need to do that ourselves
    QTextCodec * codec = QTextCodec::codecForName(encoding.toLatin1());
    I(codec);
    return codec->toUnicode(output);
}

void MonotoneTask::appendOutOfBandMessages(const QList<MessagePair> & msgs)
{
    if (msgs.size() == 0)
        return;
    oobMessages.append(msgs);
}

bool MonotoneTask::outOfBandMessagesMatch(MessageType type, const QRegExp & regex)
{
    foreach (MessagePair pair, oobMessages)
    {
        if (pair.first != type)
            continue;
        if (regex.indexIn(QString(pair.second)) >= 0)
            return true;
    }
    return false;
}

QString MonotoneTask::getLast(MessageType type)
{
    for (int i = oobMessages.size() - 1; i >= 0; i--)
    {
        if (oobMessages.at(i).first == type)
            return oobMessages.at(i).second;
    }
    return QString();
}

