/***************************************************************************
 *   Copyright (C) 2007 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#ifndef MONOTONE_THREAD_H
#define MONOTONE_THREAD_H

#include <QThread>
#include <QProcess>
#include <QQueue>
#include <QMutex>
#include <QWaitCondition>
#include <QDir>

#include "vocab.h"

class MonotoneThread : public QThread
{
    Q_OBJECT

public:
    MonotoneThread(int, const QString &, const MonotoneHandlePtr &);
    ~MonotoneThread();

    //! returns the monotone handle which has been loaded by this thread
    inline MonotoneHandlePtr getMonotoneHandle() const { return monotoneHandle; }

    //! returns the amount of tasks which are in the queue
    inline int getQueueCount() const { return queue.size(); }

    //! returns the thread's ident number
    inline int getThreadNumber() const { return threadNumber; }

    //! enqueue a new task for this thread
    int enqueueTask(const MonotoneTaskPtr &);

    //! abort task execution on this thread
    /*!
        ATTENTION: all unfinished tasks will be aborted!
        \sa taskAborted()
    */
    void abort();

    //! wait until the underlying mtn process is up and running
    bool waitForStarted();

protected:
    void run();

signals:
    //! signaled if a task has been finished
    void taskFinished(const MonotoneTaskPtr &);

    //! signaled for each task which has been aborted
    void taskAborted(const MonotoneTaskPtr &);

    //! signaled if the internal mtn process was successfully started
    void started(int);

    //! signaled if the process is stopped regularily
    //! we cannot use QThread::terminated() here because it doesn't allow
    //! the identification of the thread via an additional number
    void stopped(int);

    //! signaled if the interal mtn process was aborted (i.e. crashed)
    void aborted(int, QProcess::ProcessError, const QString &);

    //! signals newly setup / changed tickers
    void tickersChanged(const TickerMap &);

private slots:
    void cleanup(QProcess *);

private:
    bool doAbort;
    int commandNumber;
    int threadNumber;

    QString mtnBinary;
    MonotoneHandlePtr monotoneHandle;

    QQueue<MonotoneTaskPtr> queue;
    QMutex lock;

    QWaitCondition waitForTasks;

    TickerMap tickers;
};

#endif
