/***************************************************************************
 *   Copyright (C) 2006 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#include "BasicIOWriter.h"

BasicIOWriter::BasicIOWriter(const StanzaList & st) : stanzas(st)
{
    hashRegex.setPattern("^\\[([0-9a-f]{40})?\\]$");
}

BasicIOWriter::~BasicIOWriter() {}

QString BasicIOWriter::write()
{
    QString out;
    foreach (const Stanza & st, stanzas)
    {
        out.append(writeStanza(st));
    }
    return out;
}

QString BasicIOWriter::writeStanza(const Stanza & stanza)
{
    QString out;
    foreach (const StanzaEntry & en, stanza)
    {
        // FIXME: this is a somewhat quirky logic, but we have nothing better
        if (en.vals.size() == 1 && hashRegex.indexIn(en.vals.at(0)) == 0)
            out.append(writeHashLine(en));
        else
            out.append(writeValueLine(en));
    }
    out.append("\n");
    return out;
}

QString BasicIOWriter::writeHashLine(const StanzaEntry & entry)
{
    return QString("%1 %2\n").arg(entry.sym).arg(entry.vals.at(0));
}

QString BasicIOWriter::writeValueLine(const StanzaEntry & entry)
{
    QString vals;
    foreach (const QString & val, entry.vals)
    {
        vals.append(QString(" \"%1\"").arg(escape(val)));
    }
    return QString("%1%2\n").arg(entry.sym).arg(vals);
}

QString BasicIOWriter::escape(const QString & in)
{
     QString out(in);
     out.replace("\\", "\\\\");
     out.replace("\"", "\\\"");
     return out;
}

