/***************************************************************************
 *   Copyright (C) 2007 by Thomas Keller								   *
 *   me@thomaskeller.biz												   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#ifndef DEBUG_LOG_H
#define DEBUG_LOG_H

#include <QObject>
#include <QFile>

class DebugLog : public QObject
{
public:
    // higher levels include lower ones, i.e.
    // 'warn' also prints out 'critical' and 'fatal';
    // 'debug' prints out everything we have
    enum Level {Fatal = 1, Critical, Warn, Info, Debug };
    // as we have different Levels each level's name is also
    // the name for a specific log type
    typedef Level Type;

    static QString logFilePath();

    inline static void debug(const QString & msg)
    {
#ifndef QT_NO_DEBUG
        singleton()->log(Debug, msg);
#else
        Q_UNUSED(msg);
#endif
    }

    inline static void info(const QString & msg)        { singleton()->doLog(Info, msg); }
    inline static void warn(const QString & msg)        { singleton()->doLog(Warn, msg); }
    inline static void critical(const QString & msg)    { singleton()->doLog(Critical, msg); }
    inline static void fatal(const QString & msg)       { singleton()->doLog(Fatal, msg); }
    inline static void log(Type t, const QString & msg) { singleton()->doLog(t, msg); }

private:
    DebugLog();
    ~DebugLog();

    static QStringList intro();

    QByteArray format(Type, QString);
    void doLog(Type, QString);

    QFile logFile;
    bool loggedToConsole;

    static DebugLog * singleton();
    static DebugLog * instance;
};

#endif
