/***************************************************************************
 *   Copyright (C) 2006 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#ifndef DIFF_PARSER_H
#define DIFF_PARSER_H

#include <QMap>
#include <QObject>
#include <QVector>
#include <QString>
#include <QStringList>
#include <QTextCodec>

struct DiffLine
{
    enum State { Added,
                 Removed,
                 Unchanged,
                 AddedMissingNewline,
                 RemovedMissingNewline,
                 UnchangedMissingNewline
    } state;
    QString content;
};

struct DiffHunk
{
    QList<DiffLine*> lines;
    inline ~DiffHunk()
    {
        qDeleteAll(lines);
        lines.clear();
    }
    int leftStart;
    int leftCount;
    int rightStart;
    int rightCount;
};

struct Diff
{
    QList<DiffHunk*> hunks;
    inline ~Diff()
    {
        qDeleteAll(hunks);
        hunks.clear();
    }
    bool is_binary;
    bool is_new_file;
};

typedef QMap<QString,Diff*> FileDiffs;

class DiffParser : public QObject
{
    Q_OBJECT
public:
        DiffParser(const QByteArray &, const QString &);
        ~DiffParser();
        Diff * getDiff(const QString &);
        inline FileDiffs getAllDiffs() const { return fileDiffs; }

private:
        void parse(const QByteArray &);

        FileDiffs fileDiffs;
        QTextCodec * codec;
};

#endif
