/***************************************************************************
 *   Copyright (C) 2010 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#ifndef LRU_CACHE_H
#define LRU_CACHE_H

#include "vocab.h"

#include <QMap>
#include <QMultiMap>

template <class T>
class LRUCache
{
public:
    LRUCache(int max)
    {
        I(max > 0);
        maxItems = max;
    }

    T add(const T & key, int initial = 1, int update = 1)
    {
        T removed;
        if (!contains(key))
        {
            if (size() == maxItems)
            {
                removed = removeOldest();
            }
            counter.insert(key, initial);
            revCounter.insert(initial, key);
        }
        else
        {
            int oldHeat = counter.value(key);
            counter[key] = oldHeat + update;
            revCounter.remove(oldHeat, key);
            revCounter.insert(oldHeat + update, key);
        }
        return removed;
    }

    void remove(const T & key)
    {
        if (!contains(key))
            return;

        int heat = counter.take(key);
        I(revCounter.remove(heat, key) == 1);
    }

    T removeOldest()
    {
        // the keys already come in ascending order
        QList<int> keys = revCounter.uniqueKeys();
        I(keys.size() > 0);

        int heat = keys.at(0);

        QList<T> vals = revCounter.values(heat);
        I(vals.size() > 0);

        I(counter.remove(vals.last()) == 1);
        I(revCounter.remove(heat, vals.last()) == 1);
        return vals.last();
    }

    bool contains(const T & key) const
    {
        return counter.contains(key);
    }

    int size() const
    {
        return counter.size();
    }

    void clear()
    {
        counter.clear();
        revCounter.clear();
    }

private:
    int maxItems;
    QMap<T, int> counter;
    QMultiMap<int, T> revCounter;
};

#endif

