/***************************************************************************
 *   Copyright (C) 2010 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#include "ChangeKeyPassword.h"
#include "MonotoneProcess.h"
#include "Settings.h"

#include <QMessageBox>

ChangeKeyPassword::ChangeKeyPassword(QWidget * parent) : Dialog(parent)
{
    setupUi(this);
    Dialog::init();
}

ChangeKeyPassword::~ChangeKeyPassword() {}

void ChangeKeyPassword::init(const QString & hash)
{
    keyHash = hash;
    oldPasswd->clear();
    newPasswd->clear();
    newPasswd2->clear();
}

void ChangeKeyPassword::accept()
{
    if (oldPasswd->text().size() == 0 ||
        newPasswd->text().size() == 0 ||
        newPasswd2->text().size() == 0)
    {
        QMessageBox::information(
            this,
            tr("Missing information"),
            tr("Please fill out all fields."),
            QMessageBox::Ok
        );
        return;
    }

    if (newPasswd->text().compare(newPasswd2->text()) != 0)
    {
        QMessageBox::information(
            this,
            tr("Password mismatch"),
            tr("The entered passwords did not match."),
            QMessageBox::Ok
        );
        return;
    }

    QStringList args;
    args << "passphrase" << keyHash;

    QByteArray input;
    input.append(oldPasswd->text().toUtf8());
    input.append('\n');
    input.append(newPasswd->text().toUtf8());
    input.append('\n');
    input.append(newPasswd->text().toUtf8());
    input.append('\n');

    // FIXME: this is a hack: since monotone (up to 0.46 at least) prompts
    // four times for the current passphrase, we won't get notified if the
    // key decryption failed until we virtually pressed enter a fourth time
    // luckily this doesn't change the behaviour of the successful case
    input.append('\n');

    MonotoneProcess proc;
    proc.start(args);
    proc.write(input);
    proc.waitForFinished(5000);

    QString output = proc.getBufferedOutput();
    if (!proc.successful())
    {
        if (output.indexOf("failed to decrypt old private RSA key") != -1)
        {
            QMessageBox::critical(
                this,
                tr("Error changing password"),
                tr("Wrong old password for key %1 given.").arg(keyHash),
                QMessageBox::Ok, 0, 0
            );
            return;
        }

        C(QString("error changing key passphrase: %1").arg(output));

        QMessageBox::critical(
             this,
             tr("Error changing password"),
             tr("An unknown error occurred - please check the log for details."),
             QMessageBox::Ok, 0, 0
        );
        return;
    }

    if (rememberPassword->isChecked())
    {
        Settings::addItemToMap(
           "KeyPasswords",
           keyHash,
           newPasswd->text()
        );

        QMessageBox::information(
             this,
             tr("Password saved"),
             tr("Your key password has been saved. Please note that you "
                "need to re-open the database or workspace you want to "
                "use this key with."),
             QMessageBox::Ok
        );
    }

    emit keyPasswordChanged(keyHash);
    done(QDialog::Accepted);
}

