/***************************************************************************
 *   Copyright (C) 2006 by Jean-Louis Fuchs                                 *
 *   ganwell@fangorn.ch                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#ifndef CHANGESET_BROWSER_H
#define CHANGESET_BROWSER_H

#include "ui_changeset_browser.h"
#include "Branches.h"
#include "BranchesProxyModel.h"
#include "Dialog.h"
#include "GetBranchLog.h"
#include "GetRevision.h"

#include <QCloseEvent>

class ChangesetBrowser : public Dialog, private Ui::ChangesetBrowser
{
    Q_OBJECT
public:
    ChangesetBrowser(QWidget *, const MonotoneHandlePtr &);
    ~ChangesetBrowser();

    void init();

signals:
    void revisionManifest(const QString &);
    void diffRevision(const QString &, const QString &, const QString &);
    void diffFile(const QString &, const QString &, const QString &);
    void annotateFile(const QString &, const QString &);
    void createCert(const QString &, const QString &);
    void certCreated(const QString &, const QString &, const QString &, const QString &);

public slots:
    void handleCertCreation(const QString &, const QString &, const QString &, const QString &);

protected:
    void closeEvent(QCloseEvent *);

private slots:
    void branchViewClicked(const QModelIndex &);
    void changesetViewClicked(const QModelIndex &);

    void changesetViewDoubleClicked(const QModelIndex &);
    void revisionViewDoubleClicked(const QModelIndex &);

    void changesetViewContextMenuRequested(const QModelIndexList &, const QPoint &);
    void revisionViewContextMenuRequested(const QModelIndexList &, const QPoint &);

    void readAll();
    void readMore();
    void readingStopped();

    void updateParentsList();

    void suspendBranch(const QString &);

private:
    MonotoneHandlePtr monotoneHandle;
    BranchesProxyModel * branchProxyModel;
    Branches * branchModel;
    GetBranchLog * branchLogModel;
    GetRevision * revisionModel;
    QString currentRevision;
};

#endif

