/***************************************************************************
 *   Copyright (C) 2010 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#include "ConnectServerDialog.h"
#include "MonotoneUtil.h"

#include <QPushButton>

ConnectServerDialog::ConnectServerDialog(QWidget * parent) : Dialog(parent)
{
    setupUi(this);
    Dialog::init();

    validationMessage->hide();

    timer = new QTimer(this);
    timer->setInterval(1000);

    connect(
        timer, SIGNAL(timeout()),
        this, SLOT(serverConnectionChanged())
    );

    connect(
        serverAddress, SIGNAL(textEdited(const QString &)),
        timer, SLOT(start())
    );

    serverAddress->setValidator(new QRegExpValidator(
        QRegExp("^([\\w\\-]+(\\.[\\w\\-]+)*(:\\d+)?)$"),
        this
    ));

    privateKeys->addItem(tr("<anonymous>"), QString());

    QMap<QString, QString> keys =
        MonotoneUtil::getPrivateKeyMap(MonotoneHandle::create());

    QMapIterator<QString, QString> it(keys);
    while (it.hasNext())
    {
        it.next();
        privateKeys->addItem(it.value(), it.key());
    }

    connect(
        privateKeys, SIGNAL(currentIndexChanged(int)),
        this, SLOT(serverConnectionChanged())
    );
}

void ConnectServerDialog::serverConnectionChanged()
{
    timer->stop();

    QString address = serverAddress->text();
    if (address.isEmpty())
        return;

    QString selectedKey =
        privateKeys->itemData(privateKeys->currentIndex()).toString();

    try
    {
        lastValidatedHandle.clear();
        lastValidatedHandle = MonotoneHandle::create("mtn://" + selectedKey + "@" + address);
    }
    catch (GuitoneException & e)
    {
        validationMessage->setText(e.what());
        validationMessage->show();
        buttonBox->button(QDialogButtonBox::Ok)->setEnabled(false);
        return;
    }

    validationMessage->hide();
    buttonBox->button(QDialogButtonBox::Ok)->setEnabled(true);
}

void ConnectServerDialog::accept()
{
    if (lastValidatedHandle)
    {
        emit sourceSelected(lastValidatedHandle);
        done(0);
    }
}
