/***************************************************************************
 *   Copyright (C) 2008 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#include "CreateDatabase.h"
#include "MonotoneProcess.h"
#include "MonotoneUtil.h"
#include "Settings.h"

#include <QMessageBox>
#include <QFileDialog>
#include <QFileInfo>

CreateDatabase::CreateDatabase(QWidget * parent) : Dialog(parent)
{
    setupUi(this);
    Dialog::init();

    connect(
        browseFiles, SIGNAL(clicked()),
        this, SLOT(doBrowseFiles())
    );
}

CreateDatabase::~CreateDatabase() {}

void CreateDatabase::accept()
{
    QString path = databasePath->text();

    if (path.isEmpty())
    {
        QMessageBox::warning(
            this,
            tr("Please select a save location"),
            tr("Please select a location where the database should be saved."),
            QMessageBox::Ok
        );
        return;
    }

    QFileInfo fi(path);
    bool writable = fi.exists() && fi.isWritable();

    if (!writable)
    {
        QFileInfo di(fi.dir(), ".");
        writable = di.isWritable();
    }

    if (!writable)
    {
        QMessageBox::critical(
            this,
            tr("Cannot write file"),
            tr("Cannot write to the given path - please select another location."),
            QMessageBox::Ok
        );
        return;
    }

    // the user previously accepted to overwrite this file, so we can safely
    // delete it
    if (fi.exists())
    {
        if (!QFile::remove(fi.filePath()))
        {
            QMessageBox::critical(
                this,
                tr("Cannot remove old path"),
                tr("Cannot remove old path '%1'.").arg(fi.filePath()),
                QMessageBox::Ok
            );
            return;
        }
    }

    QString filePath = fi.filePath();
    if (fi.suffix() != "mtn")
    {
        filePath += ".mtn";
    }

    QStringList params;
    params << "db" << "init" << "-d" << filePath;
    QString output;

    bool res = MonotoneProcess::singleRun(params, MonotoneHandle::create(), output);

    if (!res)
    {
        QMessageBox::critical(
            this,
            tr("Cannot create database"),
            tr("An error occured while creating the database:\n%1'.")
                .arg(MonotoneUtil::stripMtnPrefix(output)),
            QMessageBox::Ok
        );
        return;
    }

    emit databaseCreated(MonotoneHandle::create(filePath));
    done(QDialog::Accepted);
}

void CreateDatabase::doBrowseFiles()
{
    QString fileName =
        QFileDialog::getSaveFileName(this, tr("Save Database"),
                                     "database.mtn",
                                     tr("Monotone databases (*.mtn *.db)"));
    databasePath->setText(fileName);
}

