/***************************************************************************
 *   Copyright (C) 2007 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#ifndef DATABASE_DIALOG_MANAGER_H
#define DATABASE_DIALOG_MANAGER_H

#include "DialogManager.h"
#include "ChangesetBrowser.h"
#include "CheckoutRevision.h"
#include "FileDiff.h"
#include "FileHistory.h"
#include "Netsync.h"
#include "RevisionDiff.h"
#include "RevisionManifest.h"
#include "SelectRevision.h"
#include "NewProjectSetup.h"
#include "AnnotateFile.h"
#include "CreateCert.h"

class DatabaseDialogManager : public DialogManager
{
    Q_OBJECT
public:
    DatabaseDialogManager(QWidget *);
    ~DatabaseDialogManager();

    virtual void closeAllDialogs();

    //! delegated signals
signals:
    void revisionSelected(const QString &);
    void revisionCheckedOut(const MonotoneHandlePtr &);
    void newProjectCreated(const MonotoneHandlePtr &);
    void certCreated(const QString &, const QString &,
                     const QString &, const QString &);

public slots:
    void showChangesetBrowser();
    void showCheckoutRevision();
    void showFileDiff(const QString & file, const QString & baseRevision, const QString & targetRevision);
    void showFileHistory(const QString & file, const QString & startRevision);
    void showNetsync();
    void showNewProjectSetup();
    void showRevisionDiff(const QString & path, const QString & baseRevision, const QString & targetRevision);
    void showRevisionManifest(const QString & revision);
    void showSelectRevision();
    void showSelectRevision(const QString & selector);
    void showAnnotation(const QString & revision, const QString & file);
    void showCreateCert(const QString & revision, const QString & certName = QString());

protected:
    ChangesetBrowser    * changesetBrowser;
    CheckoutRevision    * checkoutRevision;
    FileDiff            * fileDiff;
    FileHistory         * fileHistory;
    Netsync             * netsync;
    RevisionDiff        * revisionDiff;
    RevisionManifest    * revisionManifest;
    SelectRevision      * selectRevision;
    NewProjectSetup     * newProjectSetup;
    AnnotateFile        * annotateFile;
    CreateCert          * createCert;
};

#endif
