/***************************************************************************
 *   Copyright (C) 2007 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#ifndef DIALOGMANAGER_H
#define DIALOGMANAGER_H

#include "About.h"
#include "Preferences.h"
#include "CreateDatabase.h"
#include "KeyManagement.h"
#include "GenerateKeypair.h"
#include "ChangeKeyPassword.h"

#include "vocab.h"

#include <QStack>

class DialogManager : public QObject
{
    Q_OBJECT
public:
    DialogManager(QWidget *);
    ~DialogManager();

    void init(const MonotoneHandlePtr &);
    virtual void closeAllDialogs();

public slots:
    void showAbout();
    void showPreferences();
    void showCreateDatabase();
    void showKeyManagement();
    void showGenerateKeypair();
    void showChangeKeyPassword(const QString & keyHash);
    void showSourcePicker(const MonotoneHandle::Type & handleType);

signals:
    void sourceSelected(const MonotoneHandlePtr &);
    void keypairGenerated(const QString &);
    void keyPasswordChanged(const QString &);
    void allDialogsClosed();

protected:
    QWidget * parentWidget() const;
    void showDialog(QDialog *);

    About             * about;
    Preferences       * preferences;
    CreateDatabase    * createDatabase;
    KeyManagement     * keyManagement;
    GenerateKeypair   * generateKeypair;
    ChangeKeyPassword * changeKeyPassword;
    QDialog           * sourcePicker;

    MonotoneHandlePtr   monotoneHandle;

private:
    QStack<QDialog *> openDialogs;

private slots:
    void dialogFinished();
};

#endif

