/***************************************************************************
 *   Copyright (C) 2006 by Ingo Maindorfer                                 *
 *   ingo@liquidcooling.de                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#ifndef KEYMANAGEMENT_H
#define KEYMANAGEMENT_H

#include "Keys.h"
#include "Dialog.h"
#include "ui_key_management.h"

#include <QMenu>
#include <QContextMenuEvent>
#include <QSortFilterProxyModel>

class KeyManagement : public Dialog, private Ui::KeyManagement
{
    Q_OBJECT
public:
    KeyManagement(QWidget *, const MonotoneHandlePtr &);
    ~KeyManagement();

public slots:
    void readKeys();

signals:
    void changeKeyPassword(const QString &);
    void generateKeypair();

private:
    Key * getKeyFromSelection() const;

    Keys * model;
    QSortFilterProxyModel * proxyModel;
    MonotoneHandlePtr monotoneHandle;

    QAction * actCopyHash;
    QAction * actCopyGivenName;
    QAction * actCopyLocalName;
    QAction * actCopyPublicKey;
    QAction * actForgetPassword;
    QAction * actChangePassword;
    QAction * actDrop;

private slots:
    void contextMenuEvent(const QModelIndexList &, const QPoint &);
    void copyKeyHashToClipboard();
    void copyGivenNameToClipboard();
    void copyLocalNameToClipboard();
    void copyPubkeyDataToClipboard();
    void forgetPassword();
    void changePassword();
    void dropKey();
};

#endif
