/***************************************************************************
 *   Copyright (C) 2010 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#include "NewProjectSetup.h"
#include "WorkspaceCreator.h"
#include "MonotoneUtil.h"
#include "Platform.h"

#include <QFileDialog>
#include <QMessageBox>

NewProjectSetup::NewProjectSetup(QWidget * parent, const MonotoneHandlePtr & handle)
    : Dialog(parent), monotoneHandle(handle)
{
    setupUi(this);
    Dialog::init();

    connect(
        selectDirectory, SIGNAL(clicked()),
        this, SLOT(doSelectDirectory())
    );
}

NewProjectSetup::~NewProjectSetup() {}

void NewProjectSetup::init()
{
    branch->clear();
    workspacePath->clear();
}

void NewProjectSetup::doSelectDirectory()
{
    QString path = QFileDialog::getExistingDirectory(
        this, tr("Select a directory for the new project...")
    );

    if (!path.isEmpty())
    {
        workspacePath->setText(path);
    }
}

void NewProjectSetup::accept()
{
    QString branchName = branch->text().trimmed();
    QStringList revs = MonotoneUtil::resolveSelector(monotoneHandle, "b:" + branchName);
    if (revs.size())
    {
        QMessageBox::warning(
            this,
            tr("Branch already exists"),
            tr("The given branch already exists."),
            QMessageBox::Ok
         );
         return;
    }

    QDir workspaceDir;
    try
    {
        workspaceDir = Platform::safeMakePath(workspacePath->text());
    }
    catch (GuitoneException & e)
    {
        QMessageBox::critical(
            this,
            tr("Could not create workspace"),
            tr("The directory could not be used / created:\n%1")
                .arg(e.what()),
            QMessageBox::Ok
         );
         return;
    }

    if (!WorkspaceCreator::createBookkeepingDirectory(workspaceDir,
                                                      monotoneHandle,
                                                      branchName))
    {
        QMessageBox::critical(
            this,
            tr("Could not create workspace"),
            tr("Could not create bookkeeping directory in '%1' "
               "- please check the log for details.")
                .arg(workspaceDir.absolutePath()),
            QMessageBox::Ok
         );
         return;
    }

    emit newProjectCreated(MonotoneHandle::create(workspaceDir.absolutePath()));
    done(0);
}

