/***************************************************************************
 *   Copyright (C) 2006 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#include "RevisionDiff.h"

#include <QHeaderView>

RevisionDiff::RevisionDiff(QWidget * parent) : Dialog(parent)
{
    setupUi(this);
    Dialog::init();

    diffView->setItemsExpandable(true);
    diffView->setRootIsDecorated(true);
    // arrow width is approx. 10px
    diffView->setIndentation(10);
    // display scroll bars if the contents exceed the widget size
    diffView->header()->setResizeMode(0, QHeaderView::Interactive);
    diffView->header()->setResizeMode(1, QHeaderView::ResizeToContents);

    connect(
        diffView, SIGNAL(doubleClicked(const QModelIndex &)),
        this, SLOT(triggerFileDiff(const QModelIndex &))
    );

    diffModel = new ContentDiff(this);

    connect(
        diffModel, SIGNAL(diffRead()),
        this, SLOT(diffRead())
    );

    diffView->setModel(diffModel);
}

RevisionDiff::~RevisionDiff()
{
    delete diffModel;
}

void RevisionDiff::init(const MonotoneHandlePtr & handle, const QString & path,
                        const QString & base, const QString & target)
{
    baseRevision = base;
    targetRevision = target;
    filePath = path;

    setWindowTitle(tr("Loading differences"));

    diffModel->readDiff(handle, filePath, base, target);
}

void RevisionDiff::triggerFileDiff(const QModelIndex & index)
{
    ListLine * line = static_cast<ListLine*>(index.internalPointer());
    if (line->isFileLineAndDiffable())
    {
        emit fileDiff(line->fileName, baseRevision, targetRevision);
    }
}

void RevisionDiff::diffRead()
{
    QString left = tr("base revision"), right = tr("workspace revision");
    QString base = diffModel->getBase(), target = diffModel->getTarget();

    if (!base.isEmpty())
    {
        left = base.left(12).append("...");
    }

    if (!target.isEmpty())
    {
        right = target.left(12).append("...");
    }

    QString title;
    if (!filePath.isEmpty() && filePath != ".")
        title = tr("Differences between %1 and %2 in %3")
            .arg(left).arg(right).arg(filePath);
    else
        title = tr("Differences between %1 and %2")
            .arg(left).arg(right);

    setWindowTitle(title);
}

