/***************************************************************************
 *   Copyright (C) 2006 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#ifndef SELECT_REVISION_H
#define SELECT_REVISION_H

#include "Dialog.h"
#include "ui_select_revision.h"
#include "Select.h"
#include "Toposort.h"
#include "Certs.h"
#include "Branches.h"
#include "Tags.h"
#include "Keys.h"

#include <QCompleter>

class SelectRevision : public Dialog, private Ui::SelectRevision
{
    Q_OBJECT
public:
    SelectRevision(QWidget *, const MonotoneHandlePtr &);
    ~SelectRevision();
    void queryRevisions(const QString &);

signals:
    void revisionSelected(const QString &);
    void revisionDiff(const QString &, const QString &, const QString &);
    void revisionManifest(const QString &);

private:
    void openManifestDialog(const QString &);

    Select     * selectorModel;
    Toposort   * sortModel;
    Certs      * certsModel;
    Branches   * branchesModel;
    Tags       * tagsModel;
    Keys       * keysModel;
    QCompleter * selectorCompleter;

    QString firstRevision;
    QString secondRevision;
    QString currentRevision;
    MonotoneHandlePtr monotoneHandle;

private slots:
    void accept();
    void triggerRevisionSearch();
    void handleInvalidSelection(QString);
    void enableDisableOkButton(const QModelIndex &);
    void setCompleter(int);
    void setFirstRevision();
    void setSecondRevision();
    void enableDiffButtons(const QModelIndex &);
    void openDiffDialog();
    void readCerts(const QModelIndex &);
    void revListDoubleClicked(const QModelIndex &);
    void contextMenuRequested(const QModelIndexList &, const QPoint &);
    void trySelectFirstRow();
};

#endif
