/***************************************************************************
 *   Copyright (C) 2007 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#ifndef UPDATE_WORKSPACE_H
#define UPDATE_WORKSPACE_H

#include "Dialog.h"
#include "ui_update_workspace.h"
#include "Select.h"
#include "Branches.h"
#include "Tags.h"
#include "MonotoneProcess.h"

#include <QCompleter>

class UpdateWorkspace : public Dialog, private Ui::UpdateWorkspace
{
    Q_OBJECT
public:
    UpdateWorkspace(QWidget *, const MonotoneHandlePtr &);
    ~UpdateWorkspace();

    void init();

public slots:
    void setSelectedRevision(const QString &);

signals:
    void beforeWorkspaceUpdate();
    void workspaceUpdated(const QString &);
    void selectRevisionFromSelector(const QString &);

protected:
    virtual void closeEvent(QCloseEvent *);

private:
    Branches          * branchesModel;
    Tags              * tagsModel;
    QCompleter        * selectorCompleter;
    MonotoneProcess   * mtnProcess;
    MonotoneHandlePtr   monotoneHandle;
    QString             selectedRevisionId;

private slots:
    void slotSelectRevision();
    void accept();
    void selectBranchOption();
    void selectTagOption();
    void selectSelectorOption();
    void processOutput(const QString &);
    void processError(const QString &);
    void processFinished(int, QProcess::ExitStatus);
};

#endif
