/***************************************************************************
 *   Copyright (C) 2007 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#include "WorkspaceDialogManager.h"
#include "MonotoneUtil.h"
#include "GuitoneCore.h"

WorkspaceDialogManager::WorkspaceDialogManager(QWidget * parent)
  : DatabaseDialogManager(parent), commitRevision(0),
    unaccountedRenames(0), updateWorkspace(0) {}

WorkspaceDialogManager::~WorkspaceDialogManager()
{
    if (commitRevision)     delete commitRevision;
    if (unaccountedRenames) delete unaccountedRenames;
    if (updateWorkspace)    delete updateWorkspace;
}

void WorkspaceDialogManager::closeAllDialogs()
{
    if (commitRevision)     commitRevision->close();
    if (unaccountedRenames) unaccountedRenames->close();
    if (updateWorkspace)    updateWorkspace->close();

    DatabaseDialogManager::closeAllDialogs();
}

void WorkspaceDialogManager::showCommitRevision()
{
    showCommitRevision(QStringList());
}

void WorkspaceDialogManager::showCommitRevision(const QStringList & paths)
{
    if (!commitRevision)
    {
        commitRevision = new CommitRevision(parentWidget(), monotoneHandle);

        connect(
            commitRevision, SIGNAL(revisionCommitted(const QString &)),
            this, SIGNAL(revisionCommitted(const QString &))
        );

        connect(
            commitRevision, SIGNAL(diffFile(const QString &)),
            this,  SLOT(showFileDiff(const QString &))
        );
    }

    commitRevision->readWorkspaceRevision(paths);
    showDialog(commitRevision);
}

void WorkspaceDialogManager::showFileDiff(const QString & file)
{
    DatabaseDialogManager::showFileDiff(file, QString(), QString());
}

void WorkspaceDialogManager::showFileHistory(const QString & file)
{
    DatabaseDialogManager::showFileHistory(
        file,
        MonotoneUtil::getBaseWorkspaceRevision(monotoneHandle)
    );
}

void WorkspaceDialogManager::checkForUnaccountedRenames()
{
    if (!unaccountedRenames)
    {
        unaccountedRenames = new UnaccountedRenames(parentWidget(), monotoneHandle);

        connect(
            unaccountedRenames, SIGNAL(renamePath(const QString &, const QString &)),
            this, SIGNAL(renamePath(const QString &, const QString &))
        );
    }

    unaccountedRenames->checkForUnaccountedRenames();
}

void WorkspaceDialogManager::showUpdateWorkspace()
{
    if (!updateWorkspace)
    {
        updateWorkspace = new UpdateWorkspace(parentWidget(), monotoneHandle);

        connect(
            updateWorkspace, SIGNAL(selectRevisionFromSelector(const QString &)),
            this, SLOT(showSelectRevision(const QString &))
        );

        connect(
            updateWorkspace, SIGNAL(beforeWorkspaceUpdate()),
            this, SIGNAL(beforeWorkspaceUpdate())
        );

        connect(
            updateWorkspace, SIGNAL(workspaceUpdated(const QString &)),
            this, SIGNAL(workspaceUpdated(const QString &))
        );

        connect(
            this, SIGNAL(revisionSelected(const QString &)),
            updateWorkspace, SLOT(setSelectedRevision(const QString &))
        );
    }

    updateWorkspace->init();
    showDialog(updateWorkspace);
}

void WorkspaceDialogManager::showAnnotation(const QString & file)
{
    DatabaseDialogManager::showAnnotation(
        MonotoneUtil::getBaseWorkspaceRevision(monotoneHandle),
        file
    );
}

