/***************************************************************************
 *   Copyright (C) 2010 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#include "vocab.h"
#include "ServerWindow.h"
#include "ServerMenuBar.h"
#include "ServerDialogManager.h"
#include "GuitoneCore.h"
#include "Settings.h"
#include "MonotoneUtil.h"

#include <QHBoxLayout>

ServerWindow::ServerWindow() : MainWindow(),
    serverAddressLabel(0), privateKeyLabel(0), revisionBrowser(0), netsyncDialog(0)
{
    setObjectName("ServerWindow");
}

ServerWindow::~ServerWindow()
{
    if (serverAddressLabel) delete serverAddressLabel;
    if (privateKeyLabel)    delete privateKeyLabel;
    if (revisionBrowser)    delete revisionBrowser;
    if (netsyncDialog)      delete netsyncDialog;
}

void ServerWindow::init()
{
    if (!menuBar)
    {
        menuBar = new ServerMenuBar(this);
    }

    if (!dialogManager)
    {
        dialogManager = new ServerDialogManager(this);
    }

      connect(
        menuBar, SIGNAL(showNetsync()),
        dialogManager, SLOT(showNetsync())
    );

    connect(
        menuBar, SIGNAL(showChangesetBrowser()),
        dialogManager, SLOT(showChangesetBrowser())
    );

    MainWindow::init();
}

void ServerWindow::setup()
{
    //
    // UI stuff
    //
    QWidget * centralWidget = new QWidget(this);

    QVBoxLayout * vboxLayout = new QVBoxLayout(centralWidget);
    QGroupBox *loadedServerGroup = new QGroupBox(tr("Server connection"), centralWidget);
    vboxLayout->addWidget(loadedServerGroup);

    QVBoxLayout * vboxLayout1 = new QVBoxLayout(loadedServerGroup);
    serverAddressLabel = new QLabel(tr("Server: %1").arg(tr("n/a")), this);
    privateKeyLabel = new QLabel(tr("Authentication: %1").arg(tr("n/a")), this);
    vboxLayout1->addWidget(serverAddressLabel);
    vboxLayout1->addWidget(privateKeyLabel);

    QGroupBox * actionGroup = new QGroupBox(tr("Actions"), centralWidget);
    QVBoxLayout * vboxLayout2 = new QVBoxLayout(actionGroup);
    vboxLayout->addWidget(actionGroup);

    QHBoxLayout * hboxLayout = new QHBoxLayout();

    revisionBrowser = new QPushButton(tr("Revision browser"), actionGroup);
    hboxLayout->addWidget(revisionBrowser);
    netsyncDialog = new QPushButton(tr("Network synchronisation"), actionGroup);
    hboxLayout->addWidget(netsyncDialog);

    vboxLayout2->addLayout(hboxLayout);

    connect(
        revisionBrowser, SIGNAL(clicked()),
        dialogManager, SLOT(showChangesetBrowser())
    );

    connect(
        netsyncDialog, SIGNAL(clicked()),
        dialogManager, SLOT(showNetsync())
    );

    setCentralWidget(centralWidget);
}

void ServerWindow::load(const MonotoneHandlePtr & handle)
{
    I(handle->getType() == MonotoneHandle::server_handle);
    APP->manager()->getThread(handle);

    monotoneHandle = handle;

    QString uri = handle->getData();
    QRegExp rx("^mtn://(?:(\\w*)@)?([\\w\\-]+(?:\\.[\\w\\-]+)*(?::\\d+)?)$");
    I(rx.indexIn(uri) >= 0);

    QString winTitle;
    serverAddressLabel->setText(tr("Server: %1").arg(rx.cap(2)));

    if (rx.cap(1).isEmpty())
    {
        privateKeyLabel->setText(tr("Authentication: %1").arg(tr("<anonymous>")));
        winTitle = tr("%1 (%2)").arg(rx.cap(2)).arg(tr("<anonymous>"));
    }
    else
    {
        QMap<QString, QString> map =
            MonotoneUtil::getPrivateKeyMap(MonotoneHandle::create());
        I(map.contains(rx.cap(1)));
        privateKeyLabel->setText(tr("Authentication: %1").arg(map.value(rx.cap(1))));
        winTitle = tr("%1 (%2...)").arg(rx.cap(2)).arg(rx.cap(1).left(8));
    }

    setWindowTitle(
        tr("%1 - server mode - guitone").arg(winTitle)
    );

    dialogManager->init(monotoneHandle);
}

