/***************************************************************************
 *   Copyright (C) 2006 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#include "vocab.h"
#include "AttributesView.h"
#include "AddEditAttribute.h"

#include <QFont>

// FIXME: this dependency is ugly, we should probably use
// Qt's row editing, but this is ugly (and complex)
// to implement as well
#include "GetAttributes.h"

#include <QHeaderView>

AttributesView::AttributesView(QWidget * parent, const QString & objectName)
    : TreeView(parent, objectName)
{
    setRootIsDecorated(false);
    setItemsExpandable(false);

    QHeaderView * headerView = header();
    headerView->setClickable(true);
    headerView->setResizeMode(QHeaderView::ResizeToContents);

    connect(
        headerView, SIGNAL(sectionClicked(int)),
        this, SLOT(sectionClicked(int))
    );

    connect(
        this, SIGNAL(contextMenuRequested(const QModelIndexList &, const QPoint &)),
        this, SLOT(menuRequested(const QModelIndexList &, const QPoint &))
    );

    connect(
        this, SIGNAL(doubleClicked(const QModelIndex &)),
        this, SLOT(aboutToEditAttribute(const QModelIndex &))
    );
}

AttributesView::~AttributesView() {}

void AttributesView::sectionClicked(int logicalIndex)
{
    // do not popup the menu if we haven't focused a valid item
    GetAttributes * attrModel = qobject_cast<GetAttributes *>(model());
    I(attrModel);
    if (!attrModel->attributesLoaded()) return;

    if (logicalIndex != 0) return;

    QHeaderView * headerView = header();
    int visualIndex = headerView->visualIndex(logicalIndex);

    int left = 0;
    int top = headerView->height();

    if (visualIndex > 0)
    {
        for (int i=0; i<visualIndex; i++)
        {
            left += headerView->sectionSize(headerView->logicalIndex(i));
        }
    }

    if (left > headerView->width())
    {
        left = headerView->width() - headerView->sectionSize(logicalIndex);
    }

    QMenu popupMenu(this);
    popupMenu.addAction(
       tr("add new attribute"), this, SLOT(addAttribute())
    );
    if (attrModel->hasDroppableAttributes())
    {
        popupMenu.addAction(
           tr("drop all attributes"), this, SLOT(dropAllAttributes())
        );
    }
    popupMenu.exec(mapToGlobal(QPoint(left, top)));
}

void AttributesView::menuRequested(const QModelIndexList & indexes, const QPoint & pt)
{
    if (indexes.size() != 1) return;

    selectedIndex = indexes.at(0);

    QMenu popupMenu(this);

    QFont activeFont;
    activeFont.setBold(true);

    popupMenu.addAction(
       tr("edit attribute"), this, SLOT(editAttribute())
    )->setFont(activeFont);

    popupMenu.addAction(
       tr("drop attribute"), this, SLOT(dropAttribute())
    );

    popupMenu.exec(pt);
}

void AttributesView::addAttribute()
{
    AddEditAttribute dlg(this);

    if (dlg.exec() == QDialog::Accepted)
    {
        GetAttributes * attrModel = qobject_cast<GetAttributes*>(model());
        if (!attrModel->setAttribute(dlg.getKey(), dlg.getValue()))
        {
            C(QString("Couldn't set attribute"));
        }
        attrModel->readAttributes();
    }
}

void AttributesView::aboutToEditAttribute(const QModelIndex & index)
{
    if (!index.isValid()) return;
    selectedIndex = index;
    editAttribute();
}

void AttributesView::editAttribute()
{
    QModelIndex keyIdx = model()->index(selectedIndex.row(), 1, QModelIndex());
    QModelIndex valIdx = model()->index(selectedIndex.row(), 2, QModelIndex());

    // UserRole is overloaded with the actual string value of the
    // key / value pair
    QString key = model()->data(keyIdx, Qt::UserRole).toString();
    QString val = model()->data(valIdx, Qt::UserRole).toString();

    AddEditAttribute dlg(this, key, val);

    if (dlg.exec() == QDialog::Accepted)
    {
        GetAttributes * attrModel = qobject_cast<GetAttributes*>(model());
        if (!attrModel->setAttribute(dlg.getKey(), dlg.getValue()))
        {
            C(QString("Couldn't set/update attribute"));
        }
        attrModel->readAttributes();
    }
}

void AttributesView::dropAttribute()
{
    GetAttributes * attrModel = qobject_cast<GetAttributes*>(model());
    QModelIndex keyIdx = attrModel->index(selectedIndex.row(), 1, QModelIndex());

    if (!attrModel->dropAttribute(model()->data(keyIdx, Qt::UserRole).toString()))
    {
        C(QString("Couldn't drop attribute"));
    }
    attrModel->readAttributes();
}

void AttributesView::dropAllAttributes()
{
    GetAttributes * attrModel = qobject_cast<GetAttributes*>(model());
    if (!attrModel->dropAllAttributes())
    {
        C(QString("Couldn't drop attributes"));
    }
    attrModel->readAttributes();
}

