/***************************************************************************
 *   Copyright (C) 2007 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#ifndef MENU_BAR_H
#define MENU_BAR_H

#include <QMenuBar>

class MenuBar : public QMenuBar
{
    Q_OBJECT
public:
    MenuBar(QWidget *);
    ~MenuBar();

public slots:
    void updateRecentLists();
    void updateWindowList(const QStringList &);

signals:
    void openDatabase();
    void openWorkspace();
    void newServerConnection();
    void loadWorkspace(const QString &);
    void loadDatabase(const QString &);
    void connectToServer(const QString &);
    void showCreateDatabase();
    void showPreferences();
    void showMonotoneManual();
    void showAbout();
    void checkForApplicationUpdates();
    void closeWindow();
    void activateWindow(int);
    void raiseAllWindows();

protected:
    QAction * actionCreate_Database;
    QAction * actionOpen_Database;
    QAction * actionOpen_Workspace;
    QAction * actionConnect_to_Server;
    QAction * actionPreferences;
    QAction * actionCheck_for_updates;
    QAction * actionClose;
    QAction * actionQuit;
    QAction * actionRaise_all_windows;
    QAction * actionWindowSeparator;
    QAction * actionMonotone_manual;
    QAction * actionAbout_Qt;
    QAction * actionAbout_guitone;

    QMenu * menuFile;
    QMenu * menuRecent_Databases;
    QMenu * menuRecent_Workspaces;
    QMenu * menuRecent_Servers;
    QMenu * menuWindow;
    QMenu * menuHelp;

private slots:
    void openRecentDatabase();
    void openRecentWorkspace();
    void connectRecentServer();
    void activateOtherWindow();
};

#endif

