/***************************************************************************
 *   Copyright (C) 2009 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#include "SearchInput.h"
#include "vocab.h"
#include <QStyle>
#include <QBoxLayout>

SearchInput::SearchInput(QWidget * parent)
    : QLineEdit(parent)
{
    setMaximumHeight(20);

#ifdef Q_WS_MAC
    setAttribute(Qt::WA_MacShowFocusRect, false);

    QFont font;
    font.setPointSize(12);
    setFont(font);
#endif

    setStyleSheet(
    "QLineEdit {                                                        "
    "  border: 1px solid palette(dark);                                 "
    "  border-radius: 9px;                                              "
    "  padding: 0 20px 0 16px;                                          "
    "  selection-background-color: palette(dark);                       "
    "  background: url(:/images/search.svg) no-repeat left center;      "
    "}                                                                  "
    );

    QBoxLayout * layout = new QBoxLayout(QBoxLayout::RightToLeft, this);
    clearButton = new QPushButton(this);
    clearButton->setStyleSheet(
    "QPushButton {                                                             "
    "  border: 0;                                                              "
    "  background: transparent no-repeat center middle url(:/images/close.svg);"
    "}                                                                         "
    );
    layout->addWidget(clearButton);
    layout->addStretch();
    layout->setContentsMargins(0, 4, 0, 4);

    connect(
        clearButton, SIGNAL(clicked()),
        this, SLOT(clear())
    );

    connect(
        clearButton, SIGNAL(clicked()),
        clearButton, SLOT(hide())
    );

    connect(
        this, SIGNAL(textChanged(const QString &)),
        this, SLOT(slotTextChanged(const QString &))
    );

    clearButton->setVisible(text().size() > 0);
}

SearchInput::~SearchInput()
{
    delete clearButton;
}

void SearchInput::slotTextChanged(const QString & text)
{
    clearButton->setVisible(text.size() > 0);
}

