/***************************************************************************
 *   Copyright (C) 2007 by Thomas Keller                                   *
 *   me@thomaskeller.biz                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#include "WorkspaceMenuBar.h"
#include "vocab.h"

WorkspaceMenuBar::WorkspaceMenuBar(QWidget * parent) : DatabaseMenuBar(parent)
{
    //
    // The View/Show menu
    //
    actionAll_files = new QAction(tr("All files"), this);
    actionAll_files->setCheckable(true);
    actionAll_files->setChecked(true);
    actionAll_files->setShortcut(tr("Alt+A"));
    actionAll_changed_files = new QAction(tr("All changed files"), this);
    actionAll_changed_files->setCheckable(true);
    actionAll_changed_files->setShortcut(tr("Alt+C"));
    actionChangedContent_files = new QAction(tr("Files with changed contents"), this);
    actionChangedContent_files->setCheckable(true);
    actionChangedContent_files->setShortcut(tr("Alt+P"));
    actionChangedAttributes_files = new QAction(tr("Files with changed attributes"), this);
    actionChangedAttributes_files->setCheckable(true);
    actionChangedAttributes_files->setShortcut(tr("Alt+T"));
    actionAdded_files = new QAction(tr("Added files"), this);
    actionAdded_files->setCheckable(true);
    actionAdded_files->setShortcut(tr("Alt+N"));
    actionRemoved_files = new QAction(tr("Removed files"), this);
    actionRemoved_files->setCheckable(true);
    actionRemoved_files->setShortcut(tr("Alt+D"));
    actionRenamed_files = new QAction(tr("Renamed files"), this);
    actionRenamed_files->setCheckable(true);
    actionRenamed_files->setShortcut(tr("Alt+R"));
    actionMissing_files = new QAction(tr("Missing files"), this);
    actionMissing_files->setCheckable(true);
    actionMissing_files->setShortcut(tr("Alt+M"));
    actionUnknown_files = new QAction(tr("Unknown files"), this);
    actionUnknown_files->setCheckable(true);
    actionUnknown_files->setShortcut(tr("Alt+U"));
    actionIgnored_files = new QAction(tr("Ignored files"), this);
    actionIgnored_files->setCheckable(true);
    actionIgnored_files->setShortcut(tr("Alt+I"));
    actionInvalid_files = new QAction(tr("Invalid files"), this);
    actionInvalid_files->setCheckable(true);
    actionInvalid_files->setShortcut(tr("Alt+V"));

    // FIXME: yeah, I know, magic values suck, but registering enums
    // to be usable in Qt's signal/slot system sucks for the moment, too
    actionAll_files->setData(QVariant(0));
    actionAll_changed_files->setData(QVariant(1));
    actionChangedContent_files->setData(QVariant(2));
    actionChangedAttributes_files->setData(QVariant(3));
    actionAdded_files->setData(QVariant(4));
    actionRemoved_files->setData(QVariant(5));
    actionRenamed_files->setData(QVariant(6));
    actionMissing_files->setData(QVariant(7));
    actionUnknown_files->setData(QVariant(8));
    actionIgnored_files->setData(QVariant(9));
    actionInvalid_files->setData(QVariant(10));

    menuShow = new QMenu(tr("Show"), this);

    menuShow->addAction(actionAll_files);
    menuShow->addSeparator();
    menuShow->addAction(actionAll_changed_files);
    menuShow->addAction(actionAdded_files);
    menuShow->addAction(actionChangedContent_files);
    menuShow->addAction(actionChangedAttributes_files);
    menuShow->addAction(actionRemoved_files);
    menuShow->addAction(actionRenamed_files);
    menuShow->addAction(actionMissing_files);
    menuShow->addAction(actionUnknown_files);
    menuShow->addAction(actionIgnored_files);
    menuShow->addSeparator();
    menuShow->addAction(actionInvalid_files);

    actionHide_ignored_files = new QAction(tr("Hide ignored files"), this);
    actionHide_ignored_files->setShortcut(tr("Ctrl+H"));
    actionExpand_tree = new QAction(tr("Expand tree"), this);
    actionExpand_tree->setShortcut(tr("Ctrl+T"));

    menuView = new QMenu(tr("View"), this);

    menuView->addAction(actionHide_ignored_files);
    menuView->addAction(actionExpand_tree);
    menuView->addSeparator();
    menuView->addAction(menuShow->menuAction());

    //
    // The workspace menu
    //
    actionUpdate_workspace = new QAction(tr("Update workspace"), this);
    actionUpdate_workspace->setShortcut(tr("Ctrl+U"));
    actionCommit_revision = new QAction(tr("Commit revision"), this);
    actionCommit_revision->setShortcut(tr("Ctrl+C"));
    actionReload_workspace = new QAction(tr("Reload workspace"), this);
    actionReload_workspace->setShortcut(tr("Ctrl+R"));
    actionFind_unaccounted_renames = new QAction(tr("Find unaccounted renames"), this);

    menuWorkspace = new QMenu(tr("Workspace"), this);

    menuWorkspace->addAction(actionUpdate_workspace);
    menuWorkspace->addAction(actionCommit_revision);
    menuWorkspace->addAction(actionReload_workspace);
    menuWorkspace->addSeparator();
    menuWorkspace->addAction(actionFind_unaccounted_renames);

    // insert the workspace menu before the database menu
    insertMenu(menuDatabase->menuAction(), menuWorkspace);

    // insert the view menu before the workspace menu
    insertMenu(menuWorkspace->menuAction(), menuView);

    // view actions
    connect(
        menuShow, SIGNAL(triggered(QAction *)),
        this, SLOT(toggleFilterFiles(QAction *))
    );

    connect(
        actionExpand_tree, SIGNAL(triggered()),
        this, SLOT(toggleExpandTree())
    );

    connect(
        actionHide_ignored_files, SIGNAL(triggered()),
        this, SLOT(toggleHideIgnoredFiles())
    );

    // workspace forwards
    connect(
        actionUpdate_workspace, SIGNAL(triggered()),
        this, SIGNAL(showUpdateWorkspace())
    );

    connect(
        actionCommit_revision, SIGNAL(triggered()),
        this, SIGNAL(showCommitRevision())
    );

    connect(
        actionReload_workspace, SIGNAL(triggered()),
        this, SIGNAL(reloadWorkspace())
    );

    connect(
        actionFind_unaccounted_renames, SIGNAL(triggered()),
        this, SIGNAL(checkForUnaccountedRenames())
    );
}

WorkspaceMenuBar::~WorkspaceMenuBar() {}

void WorkspaceMenuBar::toggleFilterFiles(QAction * act)
{
    emit filterFiles(act->data().toInt());

    // disable any previous action and check the new action entry
    QList<QWidget*> assocWidgets = act->associatedWidgets();
    I(assocWidgets.size() > 0);

    // we assume that this action is only assigned to one widget and that
    // is the QMenu widget we need to find out all the other actions
    QList<QAction*> list = assocWidgets[0]->actions();
    for (int i=0, j=list.size(); i<j; i++)
    {
        list.at(i)->setChecked(false);
    }
    act->setChecked(true);
}

void WorkspaceMenuBar::toggleExpandTree()
{
    bool expanded = actionExpand_tree->data().toBool();

    emit expandTree(!expanded);

    actionExpand_tree->setText(
        expanded ? tr("Expand tree") : tr("Collapse tree")
    );

    actionExpand_tree->setData(QVariant(!expanded));
}

void WorkspaceMenuBar::toggleHideIgnoredFiles()
{
    bool hide = actionHide_ignored_files->data().toBool();

    emit hideIgnoredFiles(!hide);

    actionHide_ignored_files->setText(
        hide ? tr("Hide ignored files") : tr("Show ignored files")
    );

    actionHide_ignored_files->setData(QVariant(!hide));
}

